@@ -25,6 +25,9 @@
  * Async Disk IO operations.
  */
 
+#include "tscore/TSSystemState.h"
+#include "tscore/ink_hw.h"
+
 #include "P_AIO.h"
 
 #if AIO_MODE == AIO_MODE_NATIVE
@@ -39,7 +42,7 @@ int ts_config_with_inkdiskio = 0;
 /* structure to hold information about each file descriptor */
 AIO_Reqs *aio_reqs[MAX_DISKS_POSSIBLE];
 /* number of unique file descriptors in the aio_reqs array */
-volatile int num_filedes = 1;
+int num_filedes = 1;
 
 // acquire this mutex before inserting a new entry in the aio_reqs array.
 // Don't need to acquire this for searching the array
@@ -102,7 +105,7 @@ aio_stats_cb(const char * /* name ATS_UNUSED */, RecDataT data_type, RecData *da
   diff = new_val - sum;
   RecSetGlobalRawStatSum(aio_rsb, id, new_val);
   RecSetGlobalRawStatCount(aio_rsb, id, now);
-  data->rec_float = (float)diff * 1000.00 / (float)time_diff;
+  data->rec_float = static_cast<float>(diff) * 1000.00 / static_cast<float>(time_diff);
   return 0;
 }
 
@@ -143,11 +146,11 @@ ink_aio_set_callback(Continuation *callback)
 }
 
 void
-ink_aio_init(ModuleVersion v)
+ink_aio_init(ts::ModuleVersion v)
 {
-  ink_release_assert(!checkModuleVersion(v, AIO_MODULE_VERSION));
+  ink_release_assert(v.check(AIO_MODULE_INTERNAL_VERSION));
 
-  aio_rsb = RecAllocateRawStatBlock((int)AIO_STAT_COUNT);
+  aio_rsb = RecAllocateRawStatBlock(static_cast<int>(AIO_STAT_COUNT));
   RecRegisterRawStat(aio_rsb, RECT_PROCESS, "proxy.process.cache.read_per_sec", RECD_FLOAT, RECP_PERSISTENT,
                      (int)AIO_STAT_READ_PER_SEC, aio_stats_cb);
   RecRegisterRawStat(aio_rsb, RECT_PROCESS, "proxy.process.cache.write_per_sec", RECD_FLOAT, RECP_PERSISTENT,
@@ -158,7 +161,7 @@ ink_aio_init(ModuleVersion v)
                      (int)AIO_STAT_KB_WRITE_PER_SEC, aio_stats_cb);
 #if AIO_MODE != AIO_MODE_NATIVE
   memset(&aio_reqs, 0, MAX_DISKS_POSSIBLE * sizeof(AIO_Reqs *));
-  ink_mutex_init(&insert_mutex, nullptr);
+  ink_mutex_init(&insert_mutex);
 #endif
   REC_ReadConfigInteger(cache_config_threads_per_disk, "proxy.config.cache.threads_per_disk");
 #if TS_USE_LINUX_NATIVE_AIO
@@ -190,8 +193,13 @@ struct AIOThreadInfo : public Continuation {
     (void)event;
     (void)e;
 #if TS_USE_HWLOC
+#if HWLOC_API_VERSION >= 0x20000
+    hwloc_set_membind(ink_get_topology(), hwloc_topology_get_topology_nodeset(ink_get_topology()), HWLOC_MEMBIND_INTERLEAVE,
+                      HWLOC_MEMBIND_THREAD | HWLOC_MEMBIND_BYNODESET);
+#else
     hwloc_set_membind_nodeset(ink_get_topology(), hwloc_topology_get_topology_nodeset(ink_get_topology()), HWLOC_MEMBIND_INTERLEAVE,
                               HWLOC_MEMBIND_THREAD);
+#endif
 #endif
     aio_thread_main(this);
     delete this;
@@ -204,16 +212,12 @@ struct AIOThreadInfo : public Continuation {
   }
 };
 
-/* priority scheduling */
-/* Have 2 queues per file descriptor - A queue for http requests and another
-   for non-http (streaming) request. Each file descriptor has a lock
-   and condition variable associated with it. A dedicated number of threads
-   (THREADS_PER_DISK) wait on the condition variable associated with the
-   file descriptor. The cache threads try to put the request in the
-   appropriate queue. If they fail to acquire the lock, they put the
-   request in the atomic list. Requests are served in the order of
-   highest priority first. If both the queues are empty, the aio threads
-   check if there is any request on the other disks */
+/*
+  A dedicated number of threads (THREADS_PER_DISK) wait on the condition
+  variable associated with the file descriptor. The cache threads try to put
+  the request in the appropriate queue. If they fail to acquire the lock, they
+  put the request in the atomic list.
+ */
 
 /* insert  an entry for file descriptor fildes into aio_reqs */
 static AIO_Reqs *
@@ -221,15 +225,12 @@ aio_init_fildes(int fildes, int fromAPI = 0)
 {
   char thr_name[MAX_THREAD_NAME_LENGTH];
   int i;
-  AIO_Reqs *request = (AIO_Reqs *)ats_malloc(sizeof(AIO_Reqs));
-
-  memset(static_cast<void *>(request), 0, sizeof(AIO_Reqs));
+  AIO_Reqs *request = new AIO_Reqs;
 
   INK_WRITE_MEMORY_BARRIER;
 
   ink_cond_init(&request->aio_cond);
-  ink_mutex_init(&request->aio_mutex, nullptr);
-  ink_atomiclist_init(&request->aio_temp_list, "temp_list", (uintptr_t) & ((AIOCallback *)nullptr)->link);
+  ink_mutex_init(&request->aio_mutex);
 
   RecInt thread_num;
 
@@ -252,10 +253,11 @@ aio_init_fildes(int fildes, int fromAPI = 0)
 
   REC_ReadConfigInteger(stacksize, "proxy.config.thread.default.stacksize");
   for (i = 0; i < thread_num; i++) {
-    if (i == (thread_num - 1))
+    if (i == (thread_num - 1)) {
       thr_info = new AIOThreadInfo(request, 1);
-    else
+    } else {
       thr_info = new AIOThreadInfo(request, 0);
+    }
     snprintf(thr_name, MAX_THREAD_NAME_LENGTH, "[ET_AIO %d:%d]", i, fildes);
     ink_assert(eventProcessor.spawn_thread(thr_info, thr_name, stacksize));
   }
@@ -268,8 +270,7 @@ aio_init_fildes(int fildes, int fromAPI = 0)
   return request;
 }
 
-/* insert a request into either aio_todo or http_todo queue. aio_todo
-   list is kept sorted */
+/* insert a request into aio_todo queue. */
 static void
 aio_insert(AIOCallback *op, AIO_Reqs *req)
 {
@@ -277,48 +278,28 @@ aio_insert(AIOCallback *op, AIO_Reqs *req)
   num_requests++;
   req->queued++;
 #endif
-  if (op->aiocb.aio_reqprio == AIO_LOWEST_PRIORITY) // http request
-  {
-    AIOCallback *cb = (AIOCallback *)req->http_aio_todo.tail;
-    if (!cb)
-      req->http_aio_todo.push(op);
-    else
-      req->http_aio_todo.insert(op, cb);
-  } else {
-    AIOCallback *cb = (AIOCallback *)req->aio_todo.tail;
-
-    for (; cb; cb = (AIOCallback *)cb->link.prev) {
-      if (cb->aiocb.aio_reqprio >= op->aiocb.aio_reqprio) {
-        req->aio_todo.insert(op, cb);
-        return;
-      }
-    }
-
-    /* Either the queue was empty or this request has the highest priority */
-    req->aio_todo.push(op);
-  }
+  req->aio_todo.enqueue(op);
 }
 
 /* move the request from the atomic list to the queue */
 static void
 aio_move(AIO_Reqs *req)
 {
-  AIOCallback *next = nullptr, *prev = nullptr, *cb = (AIOCallback *)ink_atomiclist_popall(&req->aio_temp_list);
-  /* flip the list */
-  if (!cb)
+  if (req->aio_temp_list.empty()) {
     return;
-  while (cb->link.next) {
-    next          = (AIOCallback *)cb->link.next;
-    cb->link.next = prev;
-    prev          = cb;
-    cb            = next;
   }
-  /* fix the last pointer */
-  cb->link.next = prev;
-  for (; cb; cb = next) {
-    next          = (AIOCallback *)cb->link.next;
-    cb->link.next = nullptr;
-    cb->link.prev = nullptr;
+
+  AIOCallbackInternal *cbi;
+  SList(AIOCallbackInternal, alink) aq(req->aio_temp_list.popall());
+
+  // flip the list
+  Queue<AIOCallback> cbq;
+  while ((cbi = aq.pop())) {
+    cbq.push(cbi);
+  }
+
+  AIOCallback *cb;
+  while ((cb = cbq.pop())) {
     aio_insert(cb, req);
   }
 }
@@ -359,8 +340,9 @@ aio_queue_req(AIOCallbackInternal *op, int fromAPI = 0)
             break;
           }
         }
-        if (!req)
+        if (!req) {
           req = aio_init_fildes(op->aiocb.aio_fildes);
+        }
       }
       ink_mutex_release(&insert_mutex);
     }
@@ -381,14 +363,13 @@ aio_queue_req(AIOCallbackInternal *op, int fromAPI = 0)
 #ifdef AIO_STATS
     ink_atomic_increment(&data->num_temp, 1);
 #endif
-    ink_atomiclist_push(&req->aio_temp_list, op);
+    req->aio_temp_list.push(op);
   } else {
 /* check if any pending requests on the atomic list */
 #ifdef AIO_STATS
     ink_atomic_increment(&data->num_queue, 1);
 #endif
-    if (!INK_ATOMICLIST_EMPTY(req->aio_temp_list))
-      aio_move(req);
+    aio_move(req);
     /* now put the new request */
     aio_insert(op, req);
     ink_cond_signal(&req->aio_cond);
@@ -399,17 +380,18 @@ aio_queue_req(AIOCallbackInternal *op, int fromAPI = 0)
 static inline int
 cache_op(AIOCallbackInternal *op)
 {
-  bool read = (op->aiocb.aio_lio_opcode == LIO_READ) ? true : false;
+  bool read = (op->aiocb.aio_lio_opcode == LIO_READ);
   for (; op; op = (AIOCallbackInternal *)op->then) {
-    ink_aiocb_t *a = &op->aiocb;
+    ink_aiocb *a = &op->aiocb;
     ssize_t err, res = 0;
 
     while (a->aio_nbytes - res > 0) {
       do {
-        if (read)
-          err = pread(a->aio_fildes, ((char *)a->aio_buf) + res, a->aio_nbytes - res, a->aio_offset + res);
-        else
-          err = pwrite(a->aio_fildes, ((char *)a->aio_buf) + res, a->aio_nbytes - res, a->aio_offset + res);
+        if (read) {
+          err = pread(a->aio_fildes, (static_cast<char *>(a->aio_buf)) + res, a->aio_nbytes - res, a->aio_offset + res);
+        } else {
+          err = pwrite(a->aio_fildes, (static_cast<char *>(a->aio_buf)) + res, a->aio_nbytes - res, a->aio_offset + res);
+        }
       } while ((err < 0) && (errno == EINTR || errno == ENOBUFS || errno == ENOMEM));
       if (err <= 0) {
         Warning("cache disk operation failed %s %zd %d\n", (a->aio_lio_opcode == LIO_READ) ? "READ" : "WRITE", err, errno);
@@ -456,23 +438,23 @@ ink_aio_thread_num_set(int thread_num)
 void *
 aio_thread_main(void *arg)
 {
-  AIOThreadInfo *thr_info = (AIOThreadInfo *)arg;
-  AIO_Reqs *my_aio_req    = (AIO_Reqs *)thr_info->req;
+  AIOThreadInfo *thr_info = static_cast<AIOThreadInfo *>(arg);
+  AIO_Reqs *my_aio_req    = thr_info->req;
   AIO_Reqs *current_req   = nullptr;
   AIOCallback *op         = nullptr;
   ink_mutex_acquire(&my_aio_req->aio_mutex);
   for (;;) {
     do {
-      if (unlikely(shutdown_event_system == true)) {
+      if (TSSystemState::is_event_system_shut_down()) {
         ink_mutex_release(&my_aio_req->aio_mutex);
         return nullptr;
       }
       current_req = my_aio_req;
       /* check if any pending requests on the atomic list */
-      if (!INK_ATOMICLIST_EMPTY(my_aio_req->aio_temp_list))
-        aio_move(my_aio_req);
-      if (!(op = my_aio_req->aio_todo.pop()) && !(op = my_aio_req->http_aio_todo.pop()))
+      aio_move(my_aio_req);
+      if (!(op = my_aio_req->aio_todo.pop())) {
         break;
+      }
 #ifdef AIO_STATS
       num_requests--;
       current_req->queued--;
@@ -487,17 +469,8 @@ aio_thread_main(void *arg)
         aio_bytes_read += op->aiocb.aio_nbytes;
       }
       ink_mutex_release(&current_req->aio_mutex);
-      if (cache_op((AIOCallbackInternal *)op) <= 0) {
-        if (aio_err_callbck) {
-          AIOCallback *callback_op          = new AIOCallbackInternal();
-          callback_op->aiocb.aio_fildes     = op->aiocb.aio_fildes;
-          callback_op->aiocb.aio_lio_opcode = op->aiocb.aio_lio_opcode;
-          callback_op->mutex                = aio_err_callbck->mutex;
-          callback_op->action               = aio_err_callbck;
-          eventProcessor.schedule_imm(callback_op);
-        }
-      }
-      ink_atomic_increment((int *)&current_req->requests_queued, -1);
+      cache_op((AIOCallbackInternal *)op);
+      ink_atomic_increment(&current_req->requests_queued, -1);
 #ifdef AIO_STATS
       ink_atomic_increment((int *)&current_req->pending, -1);
 #endif
@@ -506,12 +479,12 @@ aio_thread_main(void *arg)
       op->mutex     = op->action.mutex;
       if (op->thread == AIO_CALLBACK_THREAD_AIO) {
         SCOPED_MUTEX_LOCK(lock, op->mutex, thr_info->mutex->thread_holding);
-        if (!op->action.cancelled)
-          op->action.continuation->handleEvent(AIO_EVENT_DONE, op);
-      } else if (op->thread == AIO_CALLBACK_THREAD_ANY)
-        eventProcessor.schedule_imm_signal(op);
-      else
-        op->thread->schedule_imm_signal(op);
+        op->handleEvent(EVENT_NONE, nullptr);
+      } else if (op->thread == AIO_CALLBACK_THREAD_ANY) {
+        eventProcessor.schedule_imm(op);
+      } else {
+        op->thread->schedule_imm(op);
+      }
       ink_mutex_acquire(&my_aio_req->aio_mutex);
     } while (true);
     timespec timedwait_msec = ink_hrtime_to_timespec(Thread::get_hrtime_updated() + HRTIME_MSECONDS(net_config_poll_timeout));
@@ -553,7 +526,7 @@ Lagain:
       Debug("aio", "io_getevents failed: %s (%d)", strerror(-ret), -ret);
   }
 
-  ink_aiocb_t *cbs[MAX_AIO_EVENTS];
+  ink_aiocb *cbs[MAX_AIO_EVENTS];
   int num = 0;
 
   for (; num < MAX_AIO_EVENTS && ((op = ready_list.dequeue()) != nullptr); ++num) {
@@ -571,13 +544,19 @@ Lagain:
       if (ret < 0) {
         Debug("aio", "io_submit failed: %s (%d)", strerror(-ret), -ret);
       } else {
-        Fatal("could not sumbit IOs, io_submit(%p, %d, %p) returned %d", ctx, num, cbs, ret);
+        Fatal("could not submit IOs, io_submit(%p, %d, %p) returned %d", ctx, num, cbs, ret);
       }
     }
   }
 
   while ((op = complete_list.dequeue()) != nullptr) {
-    op->handleEvent(event, e);
+    op->mutex = op->action.mutex;
+    MUTEX_TRY_LOCK(lock, op->mutex, trigger_event->ethread);
+    if (!lock.is_locked()) {
+      trigger_event->ethread->schedule_imm(op);
+    } else {
+      op->handleEvent(EVENT_NONE, nullptr);
+    }
   }
   return EVENT_CONT;
 }
@@ -585,7 +564,6 @@ Lagain:
 int
 ink_aio_read(AIOCallback *op, int /* fromAPI ATS_UNUSED */)
 {
-  op->aiocb.aio_reqprio    = AIO_DEFAULT_PRIORITY;
   op->aiocb.aio_lio_opcode = IO_CMD_PREAD;
   op->aiocb.data           = op;
   EThread *t               = this_ethread();
@@ -600,7 +578,6 @@ ink_aio_read(AIOCallback *op, int /* fromAPI ATS_UNUSED */)
 int
 ink_aio_write(AIOCallback *op, int /* fromAPI ATS_UNUSED */)
 {
-  op->aiocb.aio_reqprio    = AIO_DEFAULT_PRIORITY;
   op->aiocb.aio_lio_opcode = IO_CMD_PWRITE;
   op->aiocb.data           = op;
   EThread *t               = this_ethread();
@@ -621,7 +598,6 @@ ink_aio_readv(AIOCallback *op, int /* fromAPI ATS_UNUSED */)
   int sz          = 0;
 
   while (io) {
-    io->aiocb.aio_reqprio    = AIO_DEFAULT_PRIORITY;
     io->aiocb.aio_lio_opcode = IO_CMD_PREAD;
     io->aiocb.data           = io;
 #ifdef HAVE_EVENTFD
@@ -652,7 +628,6 @@ ink_aio_writev(AIOCallback *op, int /* fromAPI ATS_UNUSED */)
   int sz          = 0;
 
   while (io) {
-    io->aiocb.aio_reqprio    = AIO_DEFAULT_PRIORITY;
     io->aiocb.aio_lio_opcode = IO_CMD_PWRITE;
     io->aiocb.data           = io;
 #ifdef HAVE_EVENTFD
