@@ -25,8 +25,6 @@
 
 #include "HttpCacheSM.h" //Added to get the scope of HttpCacheSM object.
 
-extern int cache_config_compatibility_4_2_0_fixup;
-
 Action *
 Cache::open_read(Continuation *cont, const CacheKey *key, CacheFragType type, const char *hostname, int host_len)
 {
@@ -93,7 +91,7 @@ Lcallreturn:
 }
 
 Action *
-Cache::open_read(Continuation *cont, const CacheKey *key, CacheHTTPHdr *request, OverridableHttpConfigParams *params,
+Cache::open_read(Continuation *cont, const CacheKey *key, CacheHTTPHdr *request, const OverridableHttpConfigParams *params,
                  CacheFragType type, const char *hostname, int host_len)
 {
   if (!CacheProcessor::IsCacheReady(type)) {
@@ -172,7 +170,7 @@ CacheVC::load_http_info(CacheHTTPInfoVector *info, Doc *doc, RefCountObj *block_
   if (!this->f.doc_from_ram_cache && // ram cache is always already fixed up.
                                      // If this is an old object, the object version will be old or 0, in either case this is
                                      // correct. Forget the 4.2 compatibility, always update older versioned objects.
-      VersionNumber(doc->v_major, doc->v_minor) < CACHE_DB_VERSION) {
+      ts::VersionNumber(doc->v_major, doc->v_minor) < CACHE_DB_VERSION) {
     for (int i = info->xcount - 1; i >= 0; --i) {
       info->data(i).alternate.m_alt->m_response_hdr.m_mime->recompute_accelerators_and_presence_bits();
       info->data(i).alternate.m_alt->m_request_hdr.m_mime->recompute_accelerators_and_presence_bits();
@@ -224,7 +222,7 @@ CacheVC::openReadChooseWriter(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSE
     }
     // check if all the writers who came before this reader have
     // set the http_info.
-    for (w = (CacheVC *)od->writers.head; w; w = (CacheVC *)w->opendir_link.next) {
+    for (w = static_cast<CacheVC *>(od->writers.head); w; w = static_cast<CacheVC *>(w->opendir_link.next)) {
       if (w->start_time > start_time || w->closed < 0) {
         continue;
       }
@@ -278,7 +276,7 @@ CacheVC::openReadChooseWriter(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSE
       alternate_index = 0;
     }
     CacheHTTPInfo *obj = vector.get(alternate_index);
-    for (w = (CacheVC *)od->writers.head; w; w = (CacheVC *)w->opendir_link.next) {
+    for (w = static_cast<CacheVC *>(od->writers.head); w; w = static_cast<CacheVC *>(w->opendir_link.next)) {
       if (obj->m_alt == w->alternate.m_alt) {
         write_vc = w;
         break;
@@ -423,7 +421,7 @@ CacheVC::openReadFromWriter(int event, Event *e)
         // the resident alternate is being updated and its a
         // header only update. The first_buf of the writer has the
         // document body.
-        Doc *doc   = (Doc *)write_vc->first_buf->data();
+        Doc *doc   = reinterpret_cast<Doc *>(write_vc->first_buf->data());
         writer_buf = new_IOBufferBlock(write_vc->first_buf, doc->data_len(), doc->prefix_len());
         MUTEX_RELEASE(writer_lock);
         ink_assert(doc_len == doc->data_len());
@@ -489,7 +487,7 @@ CacheVC::openReadFromWriterMain(int /* event ATS_UNUSED */, Event * /* e ATS_UNU
   if (ntodo <= 0) {
     return EVENT_CONT;
   }
-  if (length < ((int64_t)doc_len) - vio.ndone) {
+  if (length < (static_cast<int64_t>(doc_len)) - vio.ndone) {
     DDebug("cache_read_agg", "truncation %X", first_key.slice32(1));
     if (is_action_tag_set("cache")) {
       ink_release_assert(false);
@@ -499,7 +497,7 @@ CacheVC::openReadFromWriterMain(int /* event ATS_UNUSED */, Event * /* e ATS_UNU
   }
   /* its possible that the user did a do_io_close before
      openWriteWriteDone was called. */
-  if (length > ((int64_t)doc_len) - vio.ndone) {
+  if (length > (static_cast<int64_t>(doc_len)) - vio.ndone) {
     int64_t skip_bytes = length - (doc_len - vio.ndone);
     iobufferblock_skip(writer_buf.get(), &writer_offset, &length, skip_bytes);
   }
@@ -507,7 +505,7 @@ CacheVC::openReadFromWriterMain(int /* event ATS_UNUSED */, Event * /* e ATS_UNU
   if (bytes > vio.ntodo()) {
     bytes = vio.ntodo();
   }
-  if (vio.ndone >= (int64_t)doc_len) {
+  if (vio.ndone >= static_cast<int64_t>(doc_len)) {
     ink_assert(bytes <= 0);
     // reached the end of the document and the user still wants more
     return calluser(VC_EVENT_EOS);
@@ -570,7 +568,7 @@ CacheVC::openReadReadDone(int event, Event *e)
     if (last_collision &&     // no missed lock
         dir_valid(vol, &dir)) // object still valid
     {
-      doc = (Doc *)buf->data();
+      doc = reinterpret_cast<Doc *>(buf->data());
       if (doc->magic != DOC_MAGIC) {
         char tmpstring[CRYPTO_HEX_SIZE];
         if (doc->magic == DOC_CORRUPT) {
@@ -645,7 +643,7 @@ int
 CacheVC::openReadMain(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSED */)
 {
   cancel_trigger();
-  Doc *doc         = (Doc *)buf->data();
+  Doc *doc         = reinterpret_cast<Doc *>(buf->data());
   int64_t ntodo    = vio.ntodo();
   int64_t bytes    = doc->len - doc_pos;
   IOBufferBlock *b = nullptr;
@@ -751,6 +749,8 @@ CacheVC::openReadMain(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSED */)
 
       doc->magic = DOC_CORRUPT;
 
+      CACHE_INCREMENT_DYN_STAT(cache_read_seek_fail_stat);
+
       CACHE_TRY_LOCK(lock, vol->mutex, mutex->thread_holding);
       if (!lock.is_locked()) {
         SET_HANDLER(&CacheVC::openReadDirDelete);
@@ -792,7 +792,7 @@ CacheVC::openReadMain(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSED */)
     return EVENT_CONT;
   }
 Lread : {
-  if (vio.ndone >= (int64_t)doc_len) {
+  if (vio.ndone >= static_cast<int64_t>(doc_len)) {
     // reached the end of the document and the user still wants more
     return calluser(VC_EVENT_EOS);
   }
@@ -876,13 +876,13 @@ CacheVC::openReadStartEarliest(int /* event ATS_UNUSED */, Event * /* e ATS_UNUS
     // an object needs to be outside the aggregation window in order to be
     // be evacuated as it is read
     if (!dir_agg_valid(vol, &dir)) {
-      // a directory entry which is nolonger valid may have been overwritten
+      // a directory entry which is no longer valid may have been overwritten
       if (!dir_valid(vol, &dir)) {
         last_collision = nullptr;
       }
       goto Lread;
     }
-    doc = (Doc *)buf->data();
+    doc = reinterpret_cast<Doc *>(buf->data());
     if (doc->magic != DOC_MAGIC) {
       char tmpstring[CRYPTO_HEX_SIZE];
       if (is_action_tag_set("cache")) {
@@ -902,7 +902,7 @@ CacheVC::openReadStartEarliest(int /* event ATS_UNUSED */, Event * /* e ATS_UNUS
       last_collision = nullptr;
       goto Lread;
     }
-    if (!(doc->key == key)) { // collisiion
+    if (!(doc->key == key)) { // collision
       goto Lread;
     }
     // success
@@ -911,7 +911,7 @@ CacheVC::openReadStartEarliest(int /* event ATS_UNUSED */, Event * /* e ATS_UNUS
     next_CacheKey(&key, &doc->key);
     vol->begin_read(this);
     if (vol->within_hit_evacuate_window(&earliest_dir) &&
-        (!cache_config_hit_evacuate_size_limit || doc_len <= (uint64_t)cache_config_hit_evacuate_size_limit)) {
+        (!cache_config_hit_evacuate_size_limit || doc_len <= static_cast<uint64_t>(cache_config_hit_evacuate_size_limit))) {
       DDebug("cache_hit_evac", "dir: %" PRId64 ", write: %" PRId64 ", phase: %d", dir_offset(&earliest_dir),
              vol->offset_to_vol_offset(vol->header->write_pos), vol->header->phase);
       f.hit_evacuate = 1;
@@ -930,7 +930,7 @@ CacheVC::openReadStartEarliest(int /* event ATS_UNUSED */, Event * /* e ATS_UNUS
     if (!f.read_from_writer_called && frag_type == CACHE_FRAG_TYPE_HTTP) {
       // don't want any writers while we are evacuating the vector
       if (!vol->open_write(this, false, 1)) {
-        Doc *doc1    = (Doc *)first_buf->data();
+        Doc *doc1    = reinterpret_cast<Doc *>(first_buf->data());
         uint32_t len = this->load_http_info(write_vector, doc1);
         ink_assert(len == doc1->hlen && write_vector->count() > 0);
         write_vector->remove(alternate_index, true);
@@ -1079,13 +1079,13 @@ CacheVC::openReadStartHead(int event, Event *e)
     // an object needs to be outside the aggregation window in order to be
     // be evacuated as it is read
     if (!dir_agg_valid(vol, &dir)) {
-      // a directory entry which is nolonger valid may have been overwritten
+      // a directory entry which is no longer valid may have been overwritten
       if (!dir_valid(vol, &dir)) {
         last_collision = nullptr;
       }
       goto Lread;
     }
-    doc = (Doc *)buf->data();
+    doc = reinterpret_cast<Doc *>(buf->data());
     if (doc->magic != DOC_MAGIC) {
       char tmpstring[CRYPTO_HEX_SIZE];
       if (is_action_tag_set("cache")) {
@@ -1123,7 +1123,7 @@ CacheVC::openReadStartHead(int event, Event *e)
         if (buf) {
           HTTPCacheAlt *alt  = reinterpret_cast<HTTPCacheAlt *>(doc->hdr());
           int32_t alt_length = 0;
-          // count should be reasonable, as vector is initialized and unlikly to be too corrupted
+          // count should be reasonable, as vector is initialized and unlikely to be too corrupted
           // by bad disk data - count should be the number of successfully unmarshalled alts.
           for (int32_t i = 0; i < vector.count(); ++i) {
             CacheHTTPInfo *info = vector.get(i);
@@ -1175,6 +1175,20 @@ CacheVC::openReadStartHead(int event, Event *e)
       } else {
         f.single_fragment = false;
       }
+
+      // Now that we have selected an alternate, validate that the content length and object size match
+      MIMEField *field = alternate.response_get()->field_find(MIME_FIELD_CONTENT_LENGTH, MIME_LEN_CONTENT_LENGTH);
+      if (field) {
+        uint64_t cl = static_cast<uint64_t>(field->value_get_int64());
+        if (cl != doc_len) {
+          Warning("OpenReadHead failed for cachekey %X : alternate content length doesn't match doc_len %" PRId64 " != %" PRId64,
+                  key.slice32(0), cl, doc_len);
+          CACHE_INCREMENT_DYN_STAT(cache_read_invalid_stat);
+          err = ECACHE_BAD_META_DATA;
+          goto Ldone;
+        }
+      }
+
     } else {
       next_CacheKey(&key, &doc->key);
       f.single_fragment = doc->single_fragment();
@@ -1195,7 +1209,7 @@ CacheVC::openReadStartHead(int event, Event *e)
     }
 
     if (vol->within_hit_evacuate_window(&dir) &&
-        (!cache_config_hit_evacuate_size_limit || doc_len <= (uint64_t)cache_config_hit_evacuate_size_limit)) {
+        (!cache_config_hit_evacuate_size_limit || doc_len <= static_cast<uint64_t>(cache_config_hit_evacuate_size_limit))) {
       DDebug("cache_hit_evac", "dir: %" PRId64 ", write: %" PRId64 ", phase: %d", dir_offset(&dir),
              vol->offset_to_vol_offset(vol->header->write_pos), vol->header->phase);
       f.hit_evacuate = 1;
