@@ -34,7 +34,7 @@
 // Common Interface impl                     //
 ///////////////////////////////////////////////
 
-ink_hrtime Thread::cur_time = ink_get_hrtime_internal();
+thread_local ink_hrtime Thread::cur_time = ink_get_hrtime_internal();
 thread_local Thread *Thread::this_thread_ptr;
 
 Thread::Thread()
