@@ -30,18 +30,70 @@
  ****************************************************************************/
 
 #include "P_SSLSNI.h"
+
+#include "PreWarmManager.h"
+
 #include "tscore/Diags.h"
 #include "tscore/SimpleTokenizer.h"
 #include "tscore/ink_memory.h"
-#include "tscpp/util/TextView.h"
 #include "tscore/I_Layout.h"
+
+#include "tscpp/util/TextView.h"
+
 #include <sstream>
+#include <utility>
 #include <pcre.h>
 
 static constexpr int OVECSIZE{30};
 
+////
+// NamedElement
+//
+NamedElement::NamedElement(NamedElement &&other)
+{
+  *this = std::move(other);
+}
+
+NamedElement &
+NamedElement::operator=(NamedElement &&other)
+{
+  if (this != &other) {
+    match = std::move(other.match);
+  }
+  return *this;
+}
+
+void
+NamedElement::set_glob_name(std::string name)
+{
+  std::string::size_type pos = 0;
+  while ((pos = name.find('.', pos)) != std::string::npos) {
+    name.replace(pos, 1, "\\.");
+    pos += 2;
+  }
+  pos = 0;
+  while ((pos = name.find('*', pos)) != std::string::npos) {
+    name.replace(pos, 1, "(.{0,})");
+  }
+  Debug("ssl_sni", "Regexed fqdn=%s", name.c_str());
+  set_regex_name(name);
+}
+
+void
+NamedElement::set_regex_name(const std::string &regex_name)
+{
+  const char *err_ptr;
+  int err_offset = 0;
+  if (!regex_name.empty()) {
+    match.reset(pcre_compile(regex_name.c_str(), PCRE_ANCHORED | PCRE_CASELESS, &err_ptr, &err_offset, nullptr));
+  }
+}
+
+////
+// SNIConfigParams
+//
 const NextHopProperty *
-SNIConfigParams::getPropertyConfig(const std::string &servername) const
+SNIConfigParams::get_property_config(const std::string &servername) const
 {
   const NextHopProperty *nps = nullptr;
   for (auto &&item : next_hop_list) {
@@ -54,12 +106,12 @@ SNIConfigParams::getPropertyConfig(const std::string &servername) const
   return nps;
 }
 
-void
-SNIConfigParams::loadSNIConfig()
+int
+SNIConfigParams::load_sni_config()
 {
-  for (auto &item : Y_sni.items) {
+  for (auto &item : yaml_sni.items) {
     auto ai = sni_action_list.emplace(sni_action_list.end());
-    ai->setGlobName(item.fqdn);
+    ai->set_glob_name(item.fqdn);
     Debug("ssl", "name: %s", item.fqdn.data());
 
     // set SNI based actions to be called in the ssl_servername_only callback
@@ -77,7 +129,14 @@ SNIConfigParams::loadSNIConfig()
       ai->actions.push_back(std::make_unique<TLSValidProtocols>(item.protocol_mask));
     }
     if (item.tunnel_destination.length() > 0) {
-      ai->actions.push_back(std::make_unique<TunnelDestination>(item.tunnel_destination, item.tunnel_type, item.tunnel_alpn));
+      ai->actions.push_back(
+        std::make_unique<TunnelDestination>(item.tunnel_destination, item.tunnel_type, item.tunnel_prewarm, item.tunnel_alpn));
+    }
+    if (!item.client_sni_policy.empty()) {
+      ai->actions.push_back(std::make_unique<OutboundSNIPolicy>(item.client_sni_policy));
+    }
+    if (item.http2_buffer_water_mark.has_value()) {
+      ai->actions.push_back(std::make_unique<HTTP2BufferWaterMark>(item.http2_buffer_water_mark.value()));
     }
 
     ai->actions.push_back(std::make_unique<SNI_IpAllow>(item.ip_allow, item.fqdn));
@@ -93,22 +152,22 @@ SNIConfigParams::loadSNIConfig()
         nps->prop.client_key_file = Layout::get()->relative_to(params->clientKeyPathOnly, item.client_key.data());
       }
 
-      params->getCTX(nps->prop.client_cert_file.c_str(),
-                     nps->prop.client_key_file.empty() ? nullptr : nps->prop.client_key_file.c_str(), params->clientCACertFilename,
-                     params->clientCACertPath);
+      auto ctx = params->getCTX(nps->prop.client_cert_file, nps->prop.client_key_file, params->clientCACertFilename,
+                                params->clientCACertPath);
+      if (ctx.get() == nullptr) {
+        return 1;
+      }
     }
 
-    nps->setGlobName(item.fqdn);
-    nps->prop.verifyServerPolicy     = item.verify_server_policy;
-    nps->prop.verifyServerProperties = item.verify_server_properties;
+    nps->set_glob_name(item.fqdn);
+    nps->prop.verify_server_policy     = item.verify_server_policy;
+    nps->prop.verify_server_properties = item.verify_server_properties;
   } // end for
+
+  return 0;
 }
 
-int SNIConfig::configid = 0;
-/*definition of member functions of SNIConfigParams*/
-SNIConfigParams::SNIConfigParams() {}
-
-std::pair<const actionVector *, ActionItem::Context>
+std::pair<const ActionVector *, ActionItem::Context>
 SNIConfigParams::get(std::string_view servername) const
 {
   int ovector[OVECSIZE];
@@ -149,7 +208,7 @@ SNIConfigParams::get(std::string_view servername) const
 }
 
 int
-SNIConfigParams::Initialize()
+SNIConfigParams::initialize()
 {
   std::string sni_filename = RecConfigReadConfigPath("proxy.config.ssl.servername.filename");
 
@@ -159,21 +218,25 @@ SNIConfigParams::Initialize()
   if (stat(sni_filename.c_str(), &sbuf) == -1 && errno == ENOENT) {
     Note("%s failed to load", sni_filename.c_str());
     Warning("Loading SNI configuration - filename: %s doesn't exist", sni_filename.c_str());
-    return 1;
+
+    return 0;
   }
 
-  ts::Errata zret = Y_sni.loader(sni_filename);
+  YamlSNIConfig yaml_sni_tmp;
+  ts::Errata zret = yaml_sni_tmp.loader(sni_filename);
   if (!zret.isOK()) {
     std::stringstream errMsg;
     errMsg << zret;
-    Error("%s failed to load: %s", sni_filename.c_str(), errMsg.str().c_str());
+    if (TSSystemState::is_initializing()) {
+      Emergency("%s failed to load: %s", sni_filename.c_str(), errMsg.str().c_str());
+    } else {
+      Error("%s failed to load: %s", sni_filename.c_str(), errMsg.str().c_str());
+    }
     return 1;
   }
+  yaml_sni = std::move(yaml_sni_tmp);
 
-  loadSNIConfig();
-  Note("%s finished loading", sni_filename.c_str());
-
-  return 0;
+  return load_sni_config();
 }
 
 SNIConfigParams::~SNIConfigParams()
@@ -181,42 +244,62 @@ SNIConfigParams::~SNIConfigParams()
   // sni_action_list and next_hop_list should cleanup with the params object
 }
 
-/*definition of member functions of SNIConfig*/
+////
+// SNIConfig
+//
+int SNIConfig::_configid = 0;
+
 void
 SNIConfig::startup()
 {
-  reconfigure();
+  if (!reconfigure()) {
+    std::string sni_filename = RecConfigReadConfigPath("proxy.config.ssl.servername.filename");
+    Fatal("failed to load %s", sni_filename.c_str());
+  }
 }
 
-void
+int
 SNIConfig::reconfigure()
 {
   Debug("ssl", "Reload SNI file");
   SNIConfigParams *params = new SNIConfigParams;
 
-  params->Initialize();
-  configid = configProcessor.set(configid, params);
+  int retStatus = params->initialize();
+  if (!retStatus) {
+    _configid = configProcessor.set(_configid, params);
+    prewarmManager.reconfigure();
+  } else {
+    delete params;
+  }
+
+  std::string sni_filename = RecConfigReadConfigPath("proxy.config.ssl.servername.filename");
+  if (!retStatus || TSSystemState::is_initializing()) {
+    Note("%s finished loading", sni_filename.c_str());
+  } else {
+    Error("%s failed to load", sni_filename.c_str());
+  }
+
+  return !retStatus;
 }
 
 SNIConfigParams *
 SNIConfig::acquire()
 {
-  return (SNIConfigParams *)configProcessor.get(configid);
+  return static_cast<SNIConfigParams *>(configProcessor.get(_configid));
 }
 
 void
 SNIConfig::release(SNIConfigParams *params)
 {
-  configProcessor.release(configid, params);
+  configProcessor.release(_configid, params);
 }
 
-// See if any of the client-side actions would trigger for this combination of servername and
-// client IP
+// See if any of the client-side actions would trigger for this combination of servername and client IP
 // host_sni_policy is an in/out parameter.  It starts with the global policy from the records.config
 // setting proxy.config.http.host_sni_policy and is possibly overridden if the sni policy
 // contains a host_sni_policy entry
 bool
-SNIConfig::TestClientAction(const char *servername, const IpEndpoint &ep, int &host_sni_policy)
+SNIConfig::test_client_action(const char *servername, const IpEndpoint &ep, int &host_sni_policy)
 {
   bool retval = false;
   SNIConfig::scoped_config params;
