@@ -355,7 +355,7 @@ QUICResumptionToken::cid() const
   return QUICTypeUtil::read_QUICConnectionId(this->_token + (1 + 20 + 4), this->_token_len - (1 + 20 + 4));
 }
 
-const ink_hrtime
+ink_hrtime
 QUICResumptionToken::expire_time() const
 {
   return QUICIntUtil::read_nbytes_as_uint(this->_token + (1 + 20), 4);
@@ -570,7 +570,7 @@ QUICPreferredAddress::store(uint8_t *buf, uint16_t &len) const
 QUICFiveTuple::QUICFiveTuple(IpEndpoint src, IpEndpoint dst, int protocol) : _source(src), _destination(dst), _protocol(protocol)
 {
   // FIXME Generate a hash code
-  this->_hash_code = src.port() + dst.port() + protocol;
+  this->_hash_code = src.network_order_port() + dst.network_order_port() + protocol;
 }
 void
 QUICFiveTuple::update(IpEndpoint src, IpEndpoint dst, int protocol)
@@ -580,7 +580,7 @@ QUICFiveTuple::update(IpEndpoint src, IpEndpoint dst, int protocol)
   this->_protocol    = protocol;
 
   // FIXME Generate a hash code
-  this->_hash_code = src.port() + dst.port() + protocol;
+  this->_hash_code = src.network_order_port() + dst.network_order_port() + protocol;
 }
 
 IpEndpoint
