@@ -22,7 +22,7 @@
  */
 
 #include <iostream>
-#include <assert.h>
+#include <cassert>
 #include <string>
 
 #include "EsiParser.h"
@@ -46,12 +46,9 @@ check_node_attr(const Attribute &attr, const char *name, const char *value)
   assert(strncmp(attr.value, value, value_len) == 0);
 }
 
-pthread_key_t threadKey;
-
 int
 main()
 {
-  pthread_key_create(&threadKey, NULL);
   Utils::init(&Debug, &Error);
 
   {
@@ -118,7 +115,7 @@ main()
   {
     cout << endl << "==================== Test 6: Invalid Quoted URL test " << endl;
     EsiParser parser("parser_test", &Debug, &Error);
-    string input_data = "<esi:include src=abc\"\"de\"f />";
+    string input_data = R"(<esi:include src=abc""de"f />)";
 
     DocNodeList node_list;
     assert(parser.parseChunk(input_data, node_list) == false);
@@ -215,7 +212,7 @@ main()
     ++list_iter;
     assert(list_iter->type == DocNode::TYPE_REMOVE);
     assert(list_iter->data_len == 0);
-    assert(list_iter->data == 0);
+    assert(list_iter->data == nullptr);
     assert((list_iter->child_nodes).size() == 0);
     ++list_iter;
     assert(list_iter->type == DocNode::TYPE_PRE);
@@ -239,7 +236,7 @@ main()
     ++list_iter;
     assert(list_iter->type == DocNode::TYPE_REMOVE);
     assert(list_iter->data_len == 0);
-    assert(list_iter->data == 0);
+    assert(list_iter->data == nullptr);
     ++list_iter;
     assert(list_iter->type == DocNode::TYPE_PRE);
     assert(list_iter->data_len == 5);
@@ -278,7 +275,7 @@ main()
     ++list_iter;
     assert(list_iter->type == DocNode::TYPE_REMOVE);
     assert(list_iter->data_len == 0);
-    assert(list_iter->data == 0);
+    assert(list_iter->data == nullptr);
     ++list_iter;
     assert(list_iter->type == DocNode::TYPE_PRE);
     assert(list_iter->data_len == 5);
@@ -306,7 +303,7 @@ main()
     ++list_iter;
     assert(list_iter->type == DocNode::TYPE_REMOVE);
     assert(list_iter->data_len == 0);
-    assert(list_iter->data == 0);
+    assert(list_iter->data == nullptr);
     ++list_iter;
     assert(list_iter->type == DocNode::TYPE_PRE);
     assert(list_iter->data_len == 4);
@@ -895,7 +892,7 @@ main()
                            "<!--esi blah--><esi:com",
                            "ment text=\"bleh\" /> <esi:remove> </esi:remove><!--esi bleh -->",
                            "<!--esi blooh--><esi:include src=url3/>",
-                           0};
+                           nullptr};
 
     DocNodeList node_list;
     for (int i = 0; lines[i]; ++i) {
