@@ -33,8 +33,8 @@ limitations under the License.
 #include <libgen.h>
 
 #include "ts/ts.h"
-#include "ts/ink_platform.h"
-#include "ts/ink_defs.h"
+#include "tscore/ink_platform.h"
+#include "tscore/ink_defs.h"
 
 static const char PLUGIN_NAME[] = "healthchecks";
 static const char SEPARATORS[]  = " \t\n";
@@ -43,11 +43,8 @@ static const char SEPARATORS[]  = " \t\n";
 #define MAX_BODY_LEN 16384
 #define FREELIST_TIMEOUT 300
 
-/* Some atomic stuff, from ATS core */
-typedef volatile void *vvoidp;
-
 static inline void *
-ink_atomic_swap_ptr(vvoidp mem, void *value)
+ink_atomic_swap_ptr(void *mem, void *value)
 {
   return __sync_lock_test_and_set((void **)mem, value);
 }
@@ -125,8 +122,9 @@ static HCDirEntry *
 find_direntry(const char *dname, HCDirEntry *dir)
 {
   while (dir) {
-    if (!strncmp(dname, dir->dname, MAX_PATH_LEN))
+    if (!strncmp(dname, dir->dname, MAX_PATH_LEN)) {
       return dir;
+    }
     dir = dir->_next;
   }
   return NULL;
@@ -139,13 +137,12 @@ setup_watchers(int fd)
   HCFileInfo *conf     = g_config;
   HCDirEntry *head_dir = NULL, *last_dir = NULL, *dir;
   char fname[MAX_PATH_LEN];
-  char *dname;
 
   while (conf) {
     conf->wd = inotify_add_watch(fd, conf->fname, IN_DELETE_SELF | IN_CLOSE_WRITE | IN_ATTRIB);
     TSDebug(PLUGIN_NAME, "Setting up a watcher for %s", conf->fname);
-    strncpy(fname, conf->fname, MAX_PATH_LEN - 1);
-    dname = dirname(fname);
+    strncpy(fname, conf->fname, MAX_PATH_LEN);
+    char *dname = dirname(fname);
     /* Make sure to only watch each directory once */
     if (!(dir = find_direntry(dname, head_dir))) {
       TSDebug(PLUGIN_NAME, "Setting up a watcher for directory %s", dname);
@@ -153,11 +150,12 @@ setup_watchers(int fd)
       memset(dir, 0, sizeof(HCDirEntry));
       strncpy(dir->dname, dname, MAX_PATH_LEN - 1);
       dir->wd = inotify_add_watch(fd, dname, IN_CREATE | IN_MOVED_FROM | IN_MOVED_TO | IN_ATTRIB);
-      if (!head_dir)
+      if (!head_dir) {
         head_dir = dir;
-      else
+      } else {
         last_dir->_next = dir;
-      last_dir          = dir;
+      }
+      last_dir = dir;
     }
     conf->dir = dir;
     conf      = conf->_next;
@@ -172,9 +170,7 @@ setup_watchers(int fd)
 static void *
 hc_thread(void *data ATS_UNUSED)
 {
-  int fd = inotify_init();
-  HCDirEntry *dirs;
-  int len;
+  int fd              = inotify_init();
   HCFileData *fl_head = NULL;
   char buffer[INOTIFY_BUFLEN];
   struct timeval last_free, now;
@@ -187,9 +183,9 @@ hc_thread(void *data ATS_UNUSED)
   while (1) {
     HCFileData *fdata = fl_head, *fdata_prev = NULL;
 
-    /* Read the inotify events, blocking until we get something */
-    len = read(fd, buffer, INOTIFY_BUFLEN);
     gettimeofday(&now, NULL);
+    /* Read the inotify events, blocking until we get something */
+    int len = read(fd, buffer, INOTIFY_BUFLEN);
 
     /* The fl_head is a linked list of previously released data entries. They
        are ordered "by time", so once we find one that is scheduled for deletion,
@@ -197,16 +193,17 @@ hc_thread(void *data ATS_UNUSED)
     while (fdata) {
       if (now.tv_sec > fdata->remove) {
         /* Now drop off the "tail" from the freelist */
-        if (fdata_prev)
+        if (fdata_prev) {
           fdata_prev->_next = NULL;
-        else
+        } else {
           fl_head = NULL;
+        }
 
         /* free() everything in the "tail" */
         do {
           HCFileData *next = fdata->_next;
 
-          TSDebug(PLUGIN_NAME, "Cleaning up entry from frelist");
+          TSDebug(PLUGIN_NAME, "Cleaning up entry from freelist");
           TSfree(fdata);
           fdata = next;
         } while (fdata);
@@ -223,9 +220,8 @@ hc_thread(void *data ATS_UNUSED)
         struct inotify_event *event = (struct inotify_event *)&buffer[i];
         HCFileInfo *finfo           = g_config;
 
-        while (
-          finfo &&
-          !((event->wd == finfo->wd) || ((event->wd == finfo->dir->wd) && !strncmp(event->name, finfo->basename, event->len)))) {
+        while (finfo && !((event->wd == finfo->wd) ||
+                          ((event->wd == finfo->dir->wd) && !strncmp(event->name, finfo->basename, event->len)))) {
           finfo = finfo->_next;
         }
         if (finfo) {
@@ -252,19 +248,12 @@ hc_thread(void *data ATS_UNUSED)
           old_data->_next  = fl_head;
           fl_head          = old_data;
         }
+        /* coverity[ -tainted_data_return] */
         i += sizeof(struct inotify_event) + event->len;
       }
     }
   }
 
-  /* Cleanup, in case we later exit this thread ... */
-  while (dirs) {
-    HCDirEntry *d = dirs;
-
-    dirs = dirs->_next;
-    TSfree(d);
-  }
-
   return NULL; /* Yeah, that never happens */
 }
 
@@ -310,7 +299,7 @@ parse_configs(const char *fname)
   } else {
     char filename[PATH_MAX + 1];
 
-    snprintf(filename, sizeof(filename), "%s/%s", fname, TSConfigDirGet());
+    snprintf(filename, sizeof(filename), "%s/%s", TSConfigDirGet(), fname);
     fd = fopen(filename, "r");
   }
 
@@ -321,28 +310,30 @@ parse_configs(const char *fname)
 
   while (!feof(fd)) {
     char *str, *save;
-    int state = 0;
     char *ok = NULL, *miss = NULL, *mime = NULL;
 
     finfo = TSmalloc(sizeof(HCFileInfo));
     memset(finfo, 0, sizeof(HCFileInfo));
 
     if (fgets(buf, sizeof(buf) - 1, fd)) {
-      str = strtok_r(buf, SEPARATORS, &save);
+      str       = strtok_r(buf, SEPARATORS, &save);
+      int state = 0;
       while (NULL != str) {
         if (strlen(str) > 0) {
           switch (state) {
           case 0:
-            if ('/' == *str)
+            if ('/' == *str) {
               ++str;
+            }
             strncpy(finfo->path, str, PATH_NAME_MAX - 1);
             finfo->p_len = strlen(finfo->path);
             break;
           case 1:
             strncpy(finfo->fname, str, MAX_PATH_LEN - 1);
             finfo->basename = strrchr(finfo->fname, '/');
-            if (finfo->basename)
+            if (finfo->basename) {
               ++(finfo->basename);
+            }
             break;
           case 2:
             mime = str;
@@ -447,15 +438,16 @@ hc_process_write(TSCont contp, TSEvent event, HCState *my_state)
     char buf[48];
     int len;
 
-    len = snprintf(buf, sizeof(buf) - 1, "Content-Length: %d\r\n\r\n", my_state->data->b_len);
+    len = snprintf(buf, sizeof(buf), "Content-Length: %d\r\n\r\n", my_state->data->b_len);
     my_state->output_bytes += add_data_to_resp(buf, len, my_state);
-    if (my_state->data->b_len > 0)
+    if (my_state->data->b_len > 0) {
       my_state->output_bytes += add_data_to_resp(my_state->data->body, my_state->data->b_len, my_state);
-    else
+    } else {
       my_state->output_bytes += add_data_to_resp("\r\n", 2, my_state);
+    }
     TSVIONBytesSet(my_state->write_vio, my_state->output_bytes);
     TSVIOReenable(my_state->write_vio);
-  } else if (TS_EVENT_VCONN_WRITE_COMPLETE) {
+  } else if (event == TS_EVENT_VCONN_WRITE_COMPLETE) {
     cleanup(contp, my_state);
   } else if (event == TS_EVENT_ERROR) {
     TSError("[healthchecks] hc_process_write: Received TS_EVENT_ERROR");
@@ -474,7 +466,7 @@ hc_process_accept(TSCont contp, HCState *my_state)
   my_state->read_vio    = TSVConnRead(my_state->net_vc, contp, my_state->req_buffer, INT64_MAX);
 }
 
-/* Imlement the server intercept */
+/* Implement the server intercept */
 static int
 hc_intercept(TSCont contp, TSEvent event, void *edata)
 {
@@ -509,9 +501,10 @@ health_check_origin(TSCont contp ATS_UNUSED, TSEvent event ATS_UNUSED, void *eda
     int path_len     = 0;
     const char *path = TSUrlPathGet(reqp, url_loc, &path_len);
 
-    /* Short circuit the / path, common case, and we won't allow healthecks on / */
-    if (!path || !path_len)
+    /* Short circuit the / path, common case, and we won't allow healthchecks on / */
+    if (!path || !path_len) {
       goto cleanup;
+    }
 
     while (info) {
       if (info->p_len == path_len && !memcmp(info->path, path, path_len)) {
@@ -521,10 +514,11 @@ health_check_origin(TSCont contp ATS_UNUSED, TSEvent event ATS_UNUSED, void *eda
       info = info->_next;
     }
 
-    if (!info)
+    if (!info) {
       goto cleanup;
+    }
 
-    TSSkipRemappingSet(txnp, 1); /* not strictly necessary, but speed is everything these days */
+    TSHttpTxnCntlSet(txnp, TS_HTTP_CNTL_SKIP_REMAPPING, true); /* not strictly necessary, but speed is everything these days */
 
     /* This is us -- register our intercept */
     icontp   = TSContCreate(hc_intercept, TSMutexCreate());
@@ -537,10 +531,12 @@ health_check_origin(TSCont contp ATS_UNUSED, TSEvent event ATS_UNUSED, void *eda
   }
 
 cleanup:
-  if (url_loc)
+  if (url_loc) {
     TSHandleMLocRelease(reqp, hdr_loc, url_loc);
-  if (hdr_loc)
+  }
+  if (hdr_loc) {
     TSHandleMLocRelease(reqp, TS_NULL_MLOC, hdr_loc);
+  }
 
   TSHttpTxnReenable(txnp, TS_EVENT_HTTP_CONTINUE);
 
@@ -554,7 +550,7 @@ TSPluginInit(int argc, const char *argv[])
   TSPluginRegistrationInfo info;
 
   if (2 != argc) {
-    TSError("[healthchecks] Must specify a configuration file.");
+    TSError("[healthchecks] Must specify a configuration file");
     return;
   }
 
@@ -563,7 +559,7 @@ TSPluginInit(int argc, const char *argv[])
   info.support_email = "dev@trafficserver.apache.org";
 
   if (TS_SUCCESS != TSPluginRegister(&info)) {
-    TSError("[healthchecks] Plugin registration failed.");
+    TSError("[healthchecks] Plugin registration failed");
     return;
   }
 
