@@ -21,20 +21,51 @@
   limitations under the License.
  */
 
-#include <stdio.h>
-#include "ts/ink_platform.h"
-#include "ts/ink_file.h"
-#include "ts/ParseRules.h"
-#include "I_RecCore.h"
-#include "ts/I_Layout.h"
+#include <cstdio>
+#include "tscore/ink_platform.h"
+#include "tscore/ink_file.h"
+#include "tscore/ParseRules.h"
+#include "records/I_RecCore.h"
+#include "tscore/I_Layout.h"
 #include "InkAPIInternal.h"
-#include "Main.h"
 #include "Plugin.h"
-#include "ts/ink_cap.h"
+#include "tscore/ink_cap.h"
+#include "tscore/Filenames.h"
+
+#define MAX_PLUGIN_ARGS 64
+
+static PluginDynamicReloadMode plugin_dynamic_reload_mode = PluginDynamicReloadMode::RELOAD_ON;
+
+bool
+isPluginDynamicReloadEnabled()
+{
+  return PluginDynamicReloadMode::RELOAD_ON == plugin_dynamic_reload_mode;
+}
+
+void
+parsePluginDynamicReloadConfig()
+{
+  int int_plugin_dynamic_reload_mode;
+
+  REC_ReadConfigInteger(int_plugin_dynamic_reload_mode, "proxy.config.plugin.dynamic_reload_mode");
+  plugin_dynamic_reload_mode = static_cast<PluginDynamicReloadMode>(int_plugin_dynamic_reload_mode);
+
+  if (plugin_dynamic_reload_mode < 0 || plugin_dynamic_reload_mode >= PluginDynamicReloadMode::RELOAD_COUNT) {
+    Warning("proxy.config.plugin.dynamic_reload_mode out of range. using default value.");
+    plugin_dynamic_reload_mode = PluginDynamicReloadMode::RELOAD_ON;
+  }
+  Note("Initialized plugin_dynamic_reload_mode: %d", plugin_dynamic_reload_mode);
+}
+
+void
+parsePluginConfig()
+{
+  parsePluginDynamicReloadConfig();
+}
 
 static const char *plugin_dir = ".";
 
-typedef void (*init_func_t)(int argc, char *argv[]);
+using init_func_t = void (*)(int, char **);
 
 // Plugin registration vars
 //
@@ -48,19 +79,16 @@ typedef void (*init_func_t)(int argc, char *argv[]);
 //      global pointer
 //
 DLL<PluginRegInfo> plugin_reg_list;
-PluginRegInfo *plugin_reg_current = NULL;
+PluginRegInfo *plugin_reg_current = nullptr;
 
-PluginRegInfo::PluginRegInfo()
-  : plugin_registered(false), plugin_path(NULL), plugin_name(NULL), vendor_name(NULL), support_email(NULL), dlh(NULL)
-{
-}
+PluginRegInfo::PluginRegInfo() = default;
 
 PluginRegInfo::~PluginRegInfo()
 {
   // We don't support unloading plugins once they are successfully loaded, so assert
   // that we don't accidentally attempt this.
   ink_release_assert(this->plugin_registered == false);
-  ink_release_assert(this->link.prev == NULL);
+  ink_release_assert(this->link.prev == nullptr);
 
   ats_free(this->plugin_path);
   ats_free(this->plugin_name);
@@ -71,8 +99,29 @@ PluginRegInfo::~PluginRegInfo()
   }
 }
 
+bool
+plugin_dso_load(const char *path, void *&handle, void *&init, std::string &error)
+{
+  handle = dlopen(path, RTLD_NOW);
+  init   = nullptr;
+  if (!handle) {
+    error.assign("unable to load '").append(path).append("': ").append(dlerror());
+    Error("%s", error.c_str());
+    return false;
+  }
+
+  init = dlsym(handle, "TSPluginInit");
+  if (!init) {
+    error.assign("unable to find TSPluginInit function in '").append(path).append("': ").append(dlerror());
+    Error("%s", error.c_str());
+    return false;
+  }
+
+  return true;
+}
+
 static bool
-plugin_load(int argc, char *argv[], bool validateOnly)
+single_plugin_init(int argc, char *argv[], bool validateOnly)
 {
   char path[PATH_NAME_MAX];
   init_func_t init;
@@ -84,7 +133,7 @@ plugin_load(int argc, char *argv[], bool validateOnly)
 
   Note("loading plugin '%s'", path);
 
-  for (PluginRegInfo *plugin_reg_temp = plugin_reg_list.head; plugin_reg_temp != NULL;
+  for (PluginRegInfo *plugin_reg_temp = plugin_reg_list.head; plugin_reg_temp != nullptr;
        plugin_reg_temp                = (plugin_reg_temp->link).next) {
     if (strcmp(plugin_reg_temp->plugin_path, path) == 0) {
       Warning("multiple loading of plugin %s", path);
@@ -99,32 +148,27 @@ plugin_load(int argc, char *argv[], bool validateOnly)
     REC_ReadConfigInteger(elevate_access, "proxy.config.plugin.load_elevated");
     ElevateAccess access(elevate_access ? ElevateAccess::FILE_PRIVILEGE : 0);
 
-    void *handle = dlopen(path, RTLD_NOW);
-    if (!handle) {
+    void *handle, *initptr = nullptr;
+    std::string error;
+    bool loaded = plugin_dso_load(path, handle, initptr, error);
+    init        = reinterpret_cast<init_func_t>(initptr);
+
+    if (!loaded) {
       if (validateOnly) {
         return false;
       }
-      Fatal("unable to load '%s': %s", path, dlerror());
+      Fatal("%s", error.c_str());
+      return false; // this line won't get called since Fatal brings down ATS
     }
 
     // Allocate a new registration structure for the
     //    plugin we're starting up
-    ink_assert(plugin_reg_current == NULL);
+    ink_assert(plugin_reg_current == nullptr);
     plugin_reg_current              = new PluginRegInfo;
     plugin_reg_current->plugin_path = ats_strdup(path);
     plugin_reg_current->dlh         = handle;
 
-    init = (init_func_t)dlsym(plugin_reg_current->dlh, "TSPluginInit");
-    if (!init) {
-      delete plugin_reg_current;
-      if (validateOnly) {
-        return false;
-      }
-      Fatal("unable to find TSPluginInit function in '%s': %s", path, dlerror());
-      return false; // this line won't get called since Fatal brings down ATS
-    }
-
-#if defined(freebsd) || defined(darwin)
+#if (!defined(kfreebsd) && defined(freebsd)) || defined(darwin)
     optreset = 1;
 #endif
 #if defined(__GLIBC__)
@@ -133,7 +177,7 @@ plugin_load(int argc, char *argv[], bool validateOnly)
     optind = 1;
 #endif
     opterr = 0;
-    optarg = NULL;
+    optarg = nullptr;
     init(argc, argv);
   } // done elevating access
 
@@ -144,7 +188,7 @@ plugin_load(int argc, char *argv[], bool validateOnly)
     return false; // this line won't get called since Fatal brings down ATS
   }
 
-  plugin_reg_current = NULL;
+  plugin_reg_current = nullptr;
 
   return true;
 }
@@ -153,10 +197,10 @@ static char *
 plugin_expand(char *arg)
 {
   RecDataT data_type;
-  char *str = NULL;
+  char *str = nullptr;
 
   if (*arg != '$') {
-    return (char *)NULL;
+    return (char *)nullptr;
   }
   // skip the $ character
   arg += 1;
@@ -171,7 +215,7 @@ plugin_expand(char *arg)
     if (RecGetRecordString_Xmalloc(arg, &str_val) != REC_ERR_OKAY) {
       goto not_found;
     }
-    return (char *)str_val;
+    return static_cast<char *>(str_val);
     break;
   }
   case RECD_FLOAT: {
@@ -179,8 +223,8 @@ plugin_expand(char *arg)
     if (RecGetRecordFloat(arg, &float_val) != REC_ERR_OKAY) {
       goto not_found;
     }
-    str = (char *)ats_malloc(128);
-    snprintf(str, 128, "%f", (float)float_val);
+    str = static_cast<char *>(ats_malloc(128));
+    snprintf(str, 128, "%f", static_cast<float>(float_val));
     return str;
     break;
   }
@@ -189,8 +233,8 @@ plugin_expand(char *arg)
     if (RecGetRecordInt(arg, &int_val) != REC_ERR_OKAY) {
       goto not_found;
     }
-    str = (char *)ats_malloc(128);
-    snprintf(str, 128, "%ld", (long int)int_val);
+    str = static_cast<char *>(ats_malloc(128));
+    snprintf(str, 128, "%ld", static_cast<long int>(int_val));
     return str;
     break;
   }
@@ -199,8 +243,8 @@ plugin_expand(char *arg)
     if (RecGetRecordCounter(arg, &count_val) != REC_ERR_OKAY) {
       goto not_found;
     }
-    str = (char *)ats_malloc(128);
-    snprintf(str, 128, "%ld", (long int)count_val);
+    str = static_cast<char *>(ats_malloc(128));
+    snprintf(str, 128, "%ld", static_cast<long int>(count_val));
     return str;
     break;
   }
@@ -210,8 +254,8 @@ plugin_expand(char *arg)
   }
 
 not_found:
-  Warning("plugin.config: unable to find parameter %s", arg);
-  return NULL;
+  Warning("%s: unable to find parameter %s", ts::filename::PLUGIN, arg);
+  return nullptr;
 }
 
 bool
@@ -219,8 +263,8 @@ plugin_init(bool validateOnly)
 {
   ats_scoped_str path;
   char line[1024], *p;
-  char *argv[64];
-  char *vars[64];
+  char *argv[MAX_PLUGIN_ARGS];
+  char *vars[MAX_PLUGIN_ARGS];
   int argc;
   int fd;
   int i;
@@ -229,15 +273,15 @@ plugin_init(bool validateOnly)
 
   if (INIT_ONCE) {
     api_init();
-    TSConfigDirGet();
-    plugin_dir = TSPluginDirGet();
+    plugin_dir = ats_stringdup(RecConfigReadPluginDir());
     INIT_ONCE  = false;
   }
 
-  path = RecConfigReadConfigPath(NULL, "plugin.config");
+  Note("%s loading ...", ts::filename::PLUGIN);
+  path = RecConfigReadConfigPath(nullptr, ts::filename::PLUGIN);
   fd   = open(path, O_RDONLY);
   if (fd < 0) {
-    Warning("unable to open plugin config file '%s': %d, %s", (const char *)path, errno, strerror(errno));
+    Warning("%s failed to load: %d, %s", ts::filename::PLUGIN, errno, strerror(errno));
     return false;
   }
 
@@ -254,7 +298,12 @@ plugin_init(bool validateOnly)
     }
 
     // not comment or blank, so rip line into tokens
-    while (1) {
+    while (true) {
+      if (argc >= MAX_PLUGIN_ARGS) {
+        Warning("Exceeded max number of args (%d) for plugin: [%s]", MAX_PLUGIN_ARGS, argc > 0 ? argv[0] : "???");
+        break;
+      }
+
       while (*p && ParseRules::is_wslfcr(*p)) {
         ++p;
       }
@@ -294,7 +343,12 @@ plugin_init(bool validateOnly)
       }
     }
 
-    retVal = plugin_load(argc, argv, validateOnly);
+    if (argc < MAX_PLUGIN_ARGS) {
+      argv[argc] = nullptr;
+    } else {
+      argv[MAX_PLUGIN_ARGS - 1] = nullptr;
+    }
+    retVal = single_plugin_init(argc, argv, validateOnly);
 
     for (i = 0; i < argc; i++) {
       ats_free(vars[i]);
@@ -302,5 +356,10 @@ plugin_init(bool validateOnly)
   }
 
   close(fd);
+  if (retVal) {
+    Note("%s finished loading", ts::filename::PLUGIN);
+  } else {
+    Error("%s failed to load", ts::filename::PLUGIN);
+  }
   return retVal;
 }
